<?php

namespace Drupal\role_paywall;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Main management class for the Role Paywall.
 */
class RolePaywallManager implements ContainerInjectionInterface, RolePaywallManagerInterface {

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Module configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $configuration;

  /**
   * Injected Plugin Manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $pluginManager;

  /**
   * List of entities blocked, indexed by entity type and then id.
   *
   * @var array
   */
  private $paywallEntities = [];

  /**
   * The entity type we are dealing with.
   *
   * @var string
   */
  private $entityType;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.role_paywall_access_rule'),
    );
  }

  /**
   * Constructs a new RolePaywallManager object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A configuration factory instance.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $plugin_manager
   *   Plugin manager for access rules.
   */
  public function __construct(ConfigFactoryInterface $config_factory, PluginManagerInterface $plugin_manager) {
    $this->configFactory = $config_factory;
    $this->pluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function setEntityType($entity_type) {
    $this->entityType = $entity_type;
    $this->configuration = $this->configFactory->get('role_paywall.settings.' . $entity_type);
  }

  /**
   * {@inheritdoc}
   */
  public function addPaywallEntity($id) {
    $paywalled_entities = $this->getPaywallEntities();
    if (!in_array($id, $paywalled_entities)) {
      $this->paywallEntities[$this->entityType][] = $id;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getPaywallEntities() {
    return (empty($this->paywallEntities[$this->entityType])) ? [] : $this->paywallEntities[$this->entityType];
  }

  /**
   * {@inheritdoc}
   */
  public function isAccessDenied($id) {
    return in_array($id, $this->getPaywallEntities());
  }

  /**
   * {@inheritdoc}
   */
  public function getPlugins() {
    $plugins = [];
    $plugin_settings = $this->configuration->get('plugins');
    uasort($plugin_settings, 'Drupal\Component\Utility\SortArray::sortByWeightElement');

    foreach ($plugin_settings as $plugin) {
      if ($plugin['enabled']) {
        $plugin_id = $plugin['plugin_id'];
        $config_object = $this->configFactory->get('role_paywall.plugin.' . $plugin_id . '.' . $this->entityType);
        $plugin_config = (empty($config_object->get('configuration'))) ? [] : $config_object->get('configuration');
        $plugins[$plugin_id] = $this->pluginManager->createInstance($plugin_id, $plugin_config);
      }
    }

    return $plugins;
  }

  /**
   * {@inheritdoc}
   */
  public function getPaywallBundles() {
    return array_values($this->configuration->get('bundles') ?: []);
  }

  /**
   * Get the field which determines if the paywall is active.
   */
  public function getPaywallActiveField($bundle) {
    $active_field = $this->configuration->get('activate_paywall_field');
    if (!empty($active_field[$bundle])) {
      return $active_field[$bundle];
    }
  }

  /**
   * Get the fields which should be hidden when access denied.
   */
  public function getPaywallHiddenFields($bundle) {
    $hidden_fields = $this->configuration->get('hidden_fields');
    if (!empty($hidden_fields[$bundle])) {
      return $hidden_fields[$bundle];
    }
  }

}
